﻿using System;
using System.Threading.Tasks;
using Windows.Devices.Geolocation;
using Windows.Media.SpeechSynthesis;
using Windows.UI;
using Windows.UI.Popups;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Media;
using Bing.Maps;
using Bing.Maps.Search;
using Bing.Speech;
using Bing.Translator;

namespace BingOverview
{
	/// <summary>
	/// An empty page that can be used on its own or navigated to within a Frame.
	/// </summary>
	public sealed partial class MainPage : Page
	{
		private const string MapCredentials = "<MapCredentials>";
		private const string ClientId = "<ClientId>";
		private const string ClientSecret = "<ClientSecret>";
		private const string BingSearchKey = "<BingSearchKey>";

		private Geolocator _geolocator;
		private SpeechRecognizer _speechRecognizer;

		public MainPage()
		{
			this.InitializeComponent();

			BingMap.Credentials = MapCredentials;
		}

		private async void Page_Loaded(object sender, RoutedEventArgs e)
		{
			// location (maps)
			_geolocator = new Geolocator();
			Geoposition position = await _geolocator.GetGeopositionAsync();
			Location location = new Location(position.Coordinate.Latitude, position.Coordinate.Longitude);
			Pushpin here = new Pushpin { Background = new SolidColorBrush(Colors.Red) };
			BingMap.Children.Add(here);
			MapLayer.SetPosition(here, location);
			BingMap.SetView(location, 10.0);


			// speech recognition
			SpeechAuthorizationParameters credentials = new SpeechAuthorizationParameters();
			credentials.ClientId = ClientId;
			credentials.ClientSecret = ClientSecret;

			_speechRecognizer = new SpeechRecognizer("en-US", credentials);
			Speech.SpeechRecognizer = _speechRecognizer;


			// translator
			Translator.ClientId = ClientId;
			Translator.ClientSecret = ClientSecret;
		}

		private async void Find_Click(object sender, RoutedEventArgs e)
		{
			Speech.Tips = new[] { "Say a place name to navigate to that location" };

			SpeechRecognitionResult result = await _speechRecognizer.RecognizeSpeechToTextAsync();
			if(result.TextConfidence != SpeechRecognitionConfidence.Rejected)
			{
				string query = result.Text.TrimEnd('.');
				Output.Text = query;

				LocationDataResponse location = await GeocodeAndPin(query);
				if(location != null)
					await BingSearch(query, location.LocationData[0].Location);
			}
		}

		private async Task<LocationDataResponse> GeocodeAndPin(string result)
		{
			GeocodeRequestOptions options = new GeocodeRequestOptions(result);
			LocationDataResponse response = await BingMap.SearchManager.GeocodeAsync(options);

			if(response != null && !response.HasError && response.LocationData.Count > 0)
			{
				await Speak(result);

				Pushpin pin = new Pushpin { Tag = response.LocationData[0].Name };
				MapLayer.SetPosition(pin, response.LocationData[0].Location);
				BingMap.Children.Add(pin);

				BingMap.SetView(response.LocationData[0].Bounds);
				return response;
			}

			await new MessageDialog("Geocode failed").ShowAsync();
			return null;
		}

		private async Task Speak(string txtSpeech)
		{
			SpeechSynthesizer synth = new SpeechSynthesizer();
			SpeechSynthesisStream stream = await synth.SynthesizeTextToStreamAsync(txtSpeech);
			Media.AutoPlay = true;
			Media.SetSource(stream, stream.ContentType);
			Media.Play();
		}

		private async Task BingSearch(string query, Location location)
		{
			Response r = await Search.GetSearchResponse(BingSearchKey, "image", query, location);
			SearchResults.ItemsSource = r.ResultSet.Results[0].Image;
		}

		private async void Translate_Click(object sender, RoutedEventArgs e)
		{
			Speech.Tips = new[] { "Say the text to translate to French" };

			SpeechRecognitionResult result = await _speechRecognizer.RecognizeSpeechToTextAsync();
			if(result.TextConfidence != SpeechRecognitionConfidence.Rejected)
			{
				Output.Text = result.Text;

				TranslationResult translate = await Translator.TranslatorApi.TranslateAsync("en", "fr", "", result.Text);
				await new MessageDialog(translate.TextTranslated).ShowAsync();
			}
		}
	}
}
